# -*- encoding: utf-8 -*-
###########################################################################
#    Module Writen to OpenERP, Open Source Management Solution
#    Copyright (C) OpenERP Venezuela (<http://openerp.com.ve>).
#    All Rights Reserved
###############Credits######################################################
#    Coded by:   Humberto Arocha humberto@openerp.com.ve
#                Angelica Barrios angelicaisabelb@gmail.com
#               Jordi Esteve <jesteve@zikzakmedia.com>
#               Javier Duran <javieredm@gmail.com>
#    Planified by: Humberto Arocha
#    Finance by: LUBCAN COL S.A.S http://www.lubcancol.com
#    Audited by: Humberto Arocha humberto@openerp.com.ve
#############################################################################
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
##############################################################################

import xml
import copy
from operator import itemgetter
import time
import datetime
from report import report_sxw
from tools import config


class account_balance(report_sxw.rml_parse):


    def __init__(self, cr, uid, name, context):
        print 'INICIO: ',datetime.datetime.now()
        super(account_balance, self).__init__(cr, uid, name, context)
        self.sum_debit = 0.00
        self.sum_credit = 0.00
        self.sum_balance = 0.00
        self.sum_debit_fy = 0.00
        self.sum_credit_fy = 0.00
        self.sum_balance_fy = 0.00
        self.date_lst = []
        self.date_lst_string = ''
        self.localcontext.update({
            'time': time,
            'lines': self.lines,
            'get_fiscalyear_text': self.get_fiscalyear_text,
            'get_periods_and_date_text': self.get_periods_and_date_text,
            'get_inf_text': self.get_informe_text,            
        })
        self.context = context


    def get_fiscalyear_text(self, form):
        """
        Returns the fiscal year text used on the report.
        """
        fiscalyear_obj = self.pool.get('account.fiscalyear')
        fiscalyear = None
        if form.get('fiscalyear'):
            fiscalyear = fiscalyear_obj.browse(self.cr, self.uid, form['fiscalyear'])
            return fiscalyear.name or fiscalyear.code
        else:
            fiscalyear = fiscalyear_obj.browse(self.cr, self.uid, fiscalyear_obj.find(self.cr, self.uid))
            return "%s*" % (fiscalyear.name or fiscalyear.code)

    def get_informe_text(self, form):
        """
        Returns the header text used on the report.
        """
        inf_type = {
            'bgen' : '               Balance General',
            'bcom' : '       Balance de Comprobacion',            
            'edogp': 'Estado de Ganancias y Perdidas' 
        }
        return inf_type[form['inf_type']]

    def get_periods_and_date_text(self, form):
        """
        Returns the text with the periods/dates used on the report.
        """
        period_obj = self.pool.get('account.period')
        periods_str = None
        fiscalyear_id = form['fiscalyear'] or fiscalyear_obj.find(self.cr, self.uid)
        period_ids = period_obj.search(self.cr, self.uid, [('fiscalyear_id','=',fiscalyear_id),('special','=',False)])
        if form['state'] in ['byperiod', 'all']:
            period_ids = form['periods'][0][2]
        periods_str = ', '.join([period.name or period.code for period in period_obj.browse(self.cr, self.uid, period_ids)])

        dates_str = None
        if form['state'] in ['bydate', 'all']:
            dates_str = self.formatLang(form['date_from'], date=True) + ' - ' + self.formatLang(form['date_to'], date=True) + ' '

        #~ if periods_str and dates_str:
            #~ return "%s / %s" % (periods_str, dates_str)
        #~ elif periods_str:
            #~ return "%s" % periods_str
        #~ elif dates_str:
            #~ return "%s" % dates_str
        #~ else:
            #~ return {'periods':periods_str, 'date':dates_str}
        return {'periods':periods_str, 'date':dates_str}


    def lines(self, form, ids={}, done=None, level=0):
        """
        Returns all the data needed for the report lines
        (account info plus debit/credit/balance in the selected period
        and the full year)
        """
        tot_bin = 0.0
        tot_deb = 0.0
        tot_crd = 0.0
        tot_eje = 0.0
        if not ids:
            ids = self.ids
        if not ids:
            return []
        if not done:
            done = {}
        if form.has_key('account_list') and form['account_list']:
            account_ids = form['account_list'][0][2]
            del form['account_list']
        res = {}
        result_acc = []
        accounts_levels = {}
        account_obj = self.pool.get('account.account')
        period_obj = self.pool.get('account.period')
        fiscalyear_obj = self.pool.get('account.fiscalyear')

        # Get the fiscal year
        fiscalyear = None
        if form.get('fiscalyear'):
            fiscalyear = fiscalyear_obj.browse(self.cr, self.uid, form['fiscalyear'])
        else:
            fiscalyear = fiscalyear_obj.browse(self.cr, self.uid, fiscalyear_obj.find(self.cr, self.uid))

        #
        # Get the accounts
        #
        def _get_children_and_consol(cr, uid, ids, level, context={}):
            aa_obj = self.pool.get('account.account')
            ids2=[]
            temp=[]
            read_data= aa_obj.read(cr, uid, ids,['id','child_id','level','type'], context)
            for data in read_data:
                if data['child_id'] and data['level'] < level and data['type']!='consolidation':
                    #ids2.append([data['id'],'Label', 'Total'])
                    ids2.append([data['id'],True, False])
                    temp=[]
                    for x in data['child_id']:
                        temp.append(x)
                    ids2 += _get_children_and_consol(cr, uid, temp, level, context)
                    ids2.append([data['id'],False,True])
                else:
                    ids2.append([data['id'],True,True])
            return ids2

        child_ids = _get_children_and_consol(self.cr, self.uid, account_ids, form['display_account_level'] and form['display_account_level'] or 100,self.context)
        if child_ids:
            account_ids = child_ids

        #
        # Calculate the FY Balance.
        # (from full fiscal year without closing periods)
        #
        ctx = self.context.copy()
        if form.get('fiscalyear'):
            # Use only the current fiscal year
            ctx['fiscalyear'] = fiscalyear.id
            ctx['periods'] = period_obj.search(self.cr, self.uid, [('fiscalyear_id','=',fiscalyear.id),'|',('special','=',False),('date_stop','<',fiscalyear.date_stop)])
        else:
            # Use all the open fiscal years
            open_fiscalyear_ids = fiscalyear_obj.search(self.cr, self.uid, [('state','=','draft')])
            ctx['periods'] = period_obj.search(self.cr, self.uid, [('fiscalyear_id','in',open_fiscalyear_ids),'|',('special','=',False),('date_stop','<',fiscalyear.date_stop)])

        fy_balance = {}
        for acc in account_obj.read(self.cr, self.uid, [x[0] for x in account_ids], ['balance'], ctx):
            fy_balance[acc['id']] = acc['balance']

        #
        # Calculate the FY Debit/Credit
        # (from full fiscal year without opening or closing periods)
        #
        ctx = self.context.copy()
        ctx['fiscalyear'] = fiscalyear.id
        ctx['periods'] = period_obj.search(self.cr, self.uid, [('fiscalyear_id','=',fiscalyear.id),('special','=',False)])

        #~ fy_debit = {}
        #~ fy_credit = {}
        #~ for acc in account_obj.read(self.cr, self.uid, [x[0] for x in account_ids], ['debit','credit','balance'], ctx):
            #~ fy_debit[acc['id']] = acc['debit']
            #~ fy_credit[acc['id']] = acc['credit']

        #
        # Calculate the period Debit/Credit
        # (from the selected period or all the non special periods in the fy)
        #
        ctx = self.context.copy()
        ctx['state'] = form['context'].get('state','all')
        ctx['fiscalyear'] = fiscalyear.id
        ctx['periods'] = period_obj.search(self.cr, self.uid, [('fiscalyear_id','=',fiscalyear.id),('special','=',False)])
        if form['state'] in ['byperiod', 'all']:
            ctx['periods'] = form['periods'][0][2]
        if form['state'] in ['bydate', 'all']:
            ctx['date_from'] = form['date_from']
            ctx['date_to'] = form['date_to']

        accounts=[]
        for aa_id in account_ids:
                new_acc = account_obj.read(self.cr, self.uid, aa_id[0], ['type','code','name','debit','credit','balance','parent_id','level'], ctx)
                #~ new_acc = account_obj.read(self.cr, self.uid, aa_id[0], ['type','code','name','debit','credit','balance','parent_id','level'], ctx)
                new_acc.update({'label':aa_id[1],'total':aa_id[2]})
                accounts.append(new_acc)
        #accounts = account_obj.read(self.cr, self.uid, account_ids, ['type','code','name','debit','credit','balance','parent_id'], ctx)
        
        def missing_period():
            ctx['fiscalyear'] = fiscalyear_obj.search(self.cr, self.uid, [('date_stop','<',fiscalyear.date_start)],order='date_stop') and \
                                fiscalyear_obj.search(self.cr, self.uid, [('date_stop','<',fiscalyear.date_start)],order='date_stop')[-1] or []
            ctx['periods'] = period_obj.search(self.cr, self.uid, [('fiscalyear_id','=',ctx['fiscalyear']),('date_stop','<',fiscalyear.date_start)])
        
        # Calculate the period initial Balance
        # (fy balance minus the balance from the start of the selected period
        #  to the end of the year)
        #
        ctx = self.context.copy()
        ctx['state'] = form['context'].get('state','all')
        ctx['fiscalyear'] = fiscalyear.id
        ctx['periods'] = period_obj.search(self.cr, self.uid, [('fiscalyear_id','=',fiscalyear.id),('special','=',False)])
        if form['state'] in ['byperiod', 'all']:
            ctx['periods'] = form['periods'][0][2]
            date_start = min([period.date_start for period in period_obj.browse(self.cr, self.uid, ctx['periods'])])
            #~ ctx['periods'] = period_obj.search(self.cr, self.uid, [('fiscalyear_id','=',fiscalyear.id),('date_start','<=',date_start),('special','=',False)])
            ctx['periods'] = period_obj.search(self.cr, self.uid, [('fiscalyear_id','=',fiscalyear.id),('date_start','<',date_start)])
            if not ctx['periods']:
                missing_period()
        if form['state'] in ['bydate', 'all']:
            if form['date_from'] > fiscalyear.date_start:
                ctx['date_from'] = fiscalyear.date_start
                ctx['date_to'] = form['date_from']
            else:
                missing_period()
        if form['state'] == 'none':
            missing_period()

        period_balanceinit = {}
        for acc in account_obj.read(self.cr, self.uid, [x[0] for x in account_ids], ['balance'], ctx):
            #~ period_balanceinit[acc['id']] = fy_balance[acc['id']] - acc['balance']
            period_balanceinit[acc['id']] = acc['balance']

        #
        # Generate the report lines (checking each account)
        #
        tot = {}        
        for account in accounts:
            account_id = account['id']

            if account_id in done:
                pass
#                continue

            done[account_id] = 1

            #
            # Calculate the account level
            #
#            parent_id = account['parent_id']
#            if parent_id:
#                if isinstance(parent_id, tuple):
#                    parent_id = parent_id[0]
#                account_level = accounts_levels.get(parent_id, 0) + 1
#            else:
#                account_level = level
            accounts_levels[account_id] = account['level']

            #
            # Check if we need to include this level
            #
            if not form['display_account_level'] or account['level'] <= form['display_account_level']:
                #
                # Copy the account values
                #
                res = {
                        'id' : account_id,
                        'type' : account['type'],
                        'code': account['code'],
                        'name': (account['total'] and not account['label']) and 'TOTAL %s'%(account['name'].upper()) or account['name'],
                        'level': account['level'],
                        'balanceinit': period_balanceinit[account_id],
                        'debit': account['debit'],
                        'credit': account['credit'],
                        'balance': period_balanceinit[account_id]+account['balance'],
                        #~ 'balance': account['balance'],
                        #~ 'balanceinit_fy': fy_balance[account_id]-fy_debit[account_id]+fy_credit[account_id],
                        #~ 'debit_fy': fy_debit[account_id],
                        #~ 'credit_fy': fy_credit[account_id],
                        #~ 'balance_fy': fy_balance[account_id],
                        'parent_id': account['parent_id'],
                        'bal_type': '',
                        'label': account['label'],
                        'total': account['total'],
                    }
                #~ print 'res: %d - %s - %s - %s '%(res['id'],res['code'],res['label'],res['total'])
                #
                # Round the values to zero if needed (-0.000001 ~= 0)
                #
                if abs(res['balance']) < 0.5 * 10**-4:
                    res['balance'] = 0.0
                #~ if abs(res['balance_fy']) < 0.5 * 10**-int(config['price_accuracy']):
                    #~ res['balance_fy'] = 0.0
                #~ if abs(res['balanceinit']) < 0.5 * 10**-int(config['price_accuracy']):
                    #~ res['balanceinit'] = 0.0
                #~ if abs(res['balanceinit_fy']) < 0.5 * 10**-int(config['price_accuracy']):
                    #~ res['balanceinit_fy'] = 0.0

                #
                # Check whether we must include this line in the report or not
                #
                if form['display_account'] == 'bal_mouvement' and account['parent_id']:
                    # Include accounts with movements
                    #~ if abs(res['balance']) >= 0.5 * 10**-int(config['price_accuracy']) \
                            #~ or abs(res['credit']) >= 0.5 * 10**-int(config['price_accuracy']) \
                            #~ or abs(res['debit']) >= 0.5 * 10**-int(config['price_accuracy']):
                    if abs(res['balance']) >= 0.5 * 10**-int(config['price_accuracy']):
                        result_acc.append(res)
                elif form['display_account'] == 'bal_solde' and account['parent_id']:
                    # Include accounts with balance
                    if abs(res['balance']) >= 0.5 * 10**-4:
                        result_acc.append(res)
                else:
                    # Include all accounts
                    result_acc.append(res)
                if form['tot_check'] and res['type'] == 'view' and res['level'] == 1 and (res['id'] not in tot):
                    tot[res['id']] = True
                    tot_bin += res['balanceinit']
                    tot_deb += res['debit']
                    tot_crd += res['credit']
                    tot_eje += res['balance']
        if form['tot_check']:
            str_label = form['lab_str']
            res2 = {
                    'type' : 'view',
                    'name': 'TOTAL %s'%(str_label),
                    'balanceinit': tot_bin,
                    'debit': tot_deb,
                    'credit': tot_crd,
                    'balance': tot_eje,
                    'label': False,
                    'total': True,
            }
            result_acc.append(res2)
        print 'FIN: ',datetime.datetime.now()
        return result_acc
report_sxw.report_sxw('report.account.account.balance.gene.4', 
                      'account.account', 
                      'addons/l10n_co_account_financial_report/report/balance_full_4_cols.rml',
                       parser=account_balance, 
                       header=False)
