# -*- encoding: utf-8 -*-
###########################################################################
#    Module Writen to OpenERP, Open Source Management Solution
#    Copyright (C) OpenERP Venezuela (<http://openerp.com.ve>).
#    All Rights Reserved
###############Credits######################################################
#    Coded by: Angelica Barrios angelicaisabelb@gmail.com
#              Humberto Arocha humberto@openerp.com.ve
#    Planified by: Humberto Arocha
#    Finance by: LUBCAN COL S.A.S http://www.lubcancol.com
#    Audited by: Humberto Arocha humberto@openerp.com.ve
#############################################################################
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
##############################################################################

from osv import osv, fields
import time
from tools import config
from tools.translate import _

class account_retention_ica(osv.osv):

    def _get_type(self, cr, uid, context=None):
        if context is None:
            context = {}
        type = context.get('type', 'in_invoice')
        return type

    def _get_journal(self, cr, uid, context):
        if context is None:
            context = {}
        type_inv = context.get('type', 'in_invoice')
        type2journal = {'out_invoice': 'ret_ica', 'in_invoice': 'ret_ica', 'out_refund': 'ret_ica', 'in_refund': 'ret_ica'}
        journal_obj = self.pool.get('account.journal')
        res = journal_obj.search(cr, uid, [('type', '=', type2journal.get(type_inv, 'ret_ica'))], limit=1)
        if res:
            return res[0]
        else:
            return False
        

    def _get_currency(self, cr, uid, context):
        user = self.pool.get('res.users').browse(cr, uid, [uid])[0]
        if user.company_id:
            return user.company_id.currency_id.id
        else:
            return self.pool.get('res.currency').search(cr, uid, [('rate','=',1.0)])[0]    
    
    
    def _get_total_rete(self, cr, uid, ids, field_name, arg, context):
        '''
        funcion para el calculo el toal de retencion
        '''
        res={}
        for i in self.browse(cr,uid,ids):
            res[i.id]=0
            cr.execute("SELECT SUM(a.suma_total)  FROM  account_retention_ica_line a   WHERE a.retention_id=%s"%(i.id))
            value=cr.fetchone()
            if value:
                res[i.id]=value[0]
        return  res
    
    
    _name = "account.retention.ica"
    _description = "Comprobante de Retencion"
    _columns = {
        'name': fields.char('Descripcion',size=128, required=False, readonly=True, states={'draft':[('readonly',False)]})                  ,         
        'period_id': fields.many2one('account.period', 'Periodo', readonly=True, states={'draft':[('readonly',False)]})                    ,
        'company_id': fields.many2one('res.company', 'Compania', required=True,  readonly=True, states={'draft':[('readonly',False)]})     ,
        'code': fields.char('Codigo', size=32, readonly=True, states={'draft':[('readonly',False)]})                                       ,
        'invoice_id': fields.many2one('account.invoice', 'Factura', required=True, ondelete='set null', select=True, readonly=True,
                      states={'draft':[('readonly',False)]})                                                                               ,
        'type': fields.selection([
            ('out_invoice','Customer Invoice')  ,
            ('in_invoice','Supplier Invoice')   ,
            ('out_refund','Customer Refund')    ,
            ('in_refund','Supplier Refund')     ,
            ],'Tipo', readonly=True, select=True, help="Tipo del Comprobante")                                                             ,
        'account_id': fields.many2one('account.account', 'Cuenta', required=True, domain="[('type', '<>', 'view')]",readonly=True,
                       states={'draft':[('readonly',False)]})                                                                              ,
        'partner_id': fields.many2one('res.partner', 'Partner',required=True,readonly=True, states={'draft':[('readonly',False)]})         ,
        'retention_line': fields.one2many('account.retention.ica.line', 'retention_id', 'Lineas de Retencion',readonly=True, 
                           states={'draft':[('readonly',False)]})                                                                          ,
        'currency_id': fields.many2one('res.currency', 'Moneda', required=True, readonly=True, states={'draft':[('readonly',False)]})      ,
        'journal_id': fields.many2one('account.journal', 'Diario', required=False, readonly=True, states={'draft':[('readonly',False)]})   ,
        'move_id': fields.many2one('account.move', 'Movimiento Contable', readonly=True, help="Asiento Contable")                          , 
        'state': fields.selection([
            ('draft','Draft')                   ,
            ('done','Done')                     ,
            ('cancel','Cancelled')
            ],'Estado', select=True, readonly=True, help="Estado del Comprobante")                                                         ,
        'date_ret': fields.date('Fecha Comprobante', help="Mantener en blanco para usar la fecha actual", readonly=True, 
                    states={'draft':[('readonly',False)]})                                                                                 ,
        'date': fields.date('Fecha',readonly=True, states={'draft':[('readonly',False)]}, help="Fecha de emision", required=True )         ,
        'total_rete':fields.function(_get_total_rete, method=True, type='float', string='Total Retencion $')                               , 
    } 
    
    
    _defaults = {
        'type': _get_type                                              ,
        'state': lambda *a: 'draft'                                    ,
        'code': lambda obj, cr, uid, context: obj.pool.get('ir.sequence').get(cr, uid, 'account.retention.ica'),
        'journal_id': _get_journal                                     ,
        'currency_id': _get_currency                                   ,
        'company_id': lambda self, cr, uid, context: \
                self.pool.get('res.users').browse(cr, uid, uid,
                    context=context).company_id.id                     ,
    }
    
    
   #metodo para saber el numero de secuencia del CODE    
    def _get_seq_code(self, cr, uid, context=None):
        if context is None:
            context={}
        pool_seq=self.pool.get('ir.sequence')
        cr.execute("select id,number_next,number_increment,prefix,suffix,padding from ir_sequence where code='account.retention.ica' and active=True")
        res = cr.dictfetchone()
        if res:
            if res['number_next']:
                return pool_seq._process(res['prefix']) + '%%0%sd' % res['padding'] % res['number_next'] + pool_seq._process(res['suffix'])
            else:
                return pool_seq._process(res['prefix']) + pool_seq._process(res['suffix'])
        return False
    
 
    
    #metodo que llama el boton de la interface retention_ica_view.xml y que realiza el asiento contable
    def action_done(self, cr, uid, ids, context={}):
        obj_ret = self.browse(cr, uid, ids)[0]
        print "Entro en el metodo para realizar el asiento contable"    
        self.action_move_create(cr, uid, ids) # genera el comprobante contable
        self.write(cr, uid, ids, {'state':'done'})
        return True
    
    #metodo que genera el comprobante contable
    def action_move_create(self, cr, uid, ids, *args):
        print "Metodo que genera el asiento contable"
        inv_obj = self.pool.get('account.invoice')
        context = {}
        linea=[]
        amount_Total=0
        for ret in self.browse(cr, uid, ids):
            acc_id = ret.account_id.id #cuenta contable
            if not ret.date_ret:#si no hay fecha de retencion
                self.write(cr, uid, [ret.id], {'date_ret':time.strftime('%Y-%m-%d')}) #se coloca la fecha actual del sistema

            period_id = ret.period_id and ret.period_id.id or False
            journal_id = ret.journal_id.id #en el caso que no tenga journal hay que validar esta parte
            
            if not period_id: #si no tiene periodo
                period_ids = self.pool.get('account.period').search(cr,uid,[('date_start','<=',ret.date_ret or time.strftime('%Y-%m-%d')),('date_stop','>=',ret.date_ret or time.strftime('%Y-%m-%d'))])
                if len(period_ids):
                    period_id = period_ids[0]
                else:
                    raise osv.except_osv(_('Warning !'), _("No se encontro un periodo fiscal para esta fecha: '%s' por favor verificar.!") % (ret.date_ret or time.strftime('%Y-%m-%d')))

            if ret.retention_line: #si hay lineas de retencion
                for line in ret.retention_line: #por cada linea de retencion se generan cuentas contables almacenadas en la variable linea
                    amount = line.suma_total
                    amount_Total+=amount
                    ret_move = self.crear_lineas_asientos(cr, uid,[ret.id],[line.id],amount, context) 
                    linea.append(ret_move)                

                writeoff_account_id = False #cuenta
                writeoff_journal_id = False #jornal
                ret_move = self.ret_and_reconcile(  cr, uid, linea, [ret.id], 
                                                    amount_Total, acc_id, period_id, journal_id, writeoff_account_id,
                                                    period_id, writeoff_journal_id, context)
                rl = {'move_id': ret_move['move_id'],}                    
                move = [(1, ret.id, rl)]    
                self.write(cr, uid, [ret.id], {'move_id':ret_move['move_id'], 'period_id':period_id}) 
        return True   
    

    #metodo que genera las account.move.line  por la unica linea en el modelo de ica 
    def crear_lineas_asientos(self, cr, uid, ids, line_ids2, pay_amount, context=None, name=''):
        print "Metodo de lineas de asientos"
        line_obj = self.pool.get('account.retention.ica.line')      
        ret = self.browse(cr, uid, ids)[0] #account.retention.ica
        if context is None:
            context = {}
        #TODO check if we can use different period for payment and the writeoff line
        assert len(line_ids2)==1, "Can only pay one line at a time"
        line = line_obj.browse(cr, uid, line_ids2[0]) #lineas de retencion del modelo             
        src_account_id = line.account_id.id #cuenta de la linea del asiento       
        # Take the seq as name for move
        #################Notas de Credito###########################
        if ret.type=="out_refund" or ret.type=="in_refund" :#para notas de credito
            types = {'out_invoice': -1, 'in_invoice': 1, 'out_refund': 1, 'in_refund': -1}
        else:
            types = {'out_invoice': 1, 'in_invoice': -1, 'out_refund': -1, 'in_refund': 1}
        direction = types[ret.type]
        #take the choosen date
        if 'date_p' in context and context['date_p']:
            date=context['date_p']
        else:
            date=time.strftime('%Y-%m-%d')   
        l1 = { 
            'debit': direction * pay_amount>0 and direction * pay_amount       ,
            'credit': direction * pay_amount<0 and - direction * pay_amount    ,
            'account_id': src_account_id                                       , #cuenta de cada linea
            'partner_id': ret.partner_id.id                                    ,
            #'product_id':line.producto_id.id                                  ,#para ica en la linea no hay producto
            'ref':ret.code                                                   ,
            'date': date                                                       ,
        }        
        if not name:
            if ret.type in ['in_invoice','in_refund']:
                name = 'COMP. RET. ICA ' + ret.code + ' Doc. '+ (ret.invoice_id.reference or '')
            else:
                name = 'COMP. RET. ICA ' + ret.code + ' Doc. '+ (ret.invoice_id.number or ''), 
        l1['name'] = name
        lines = (0, 0, l1)
        return lines
    
    
    #metodo que genera la cuenta contable para la cabecera del modelo y uno las demas lineas account.move.line 
    def ret_and_reconcile(self, cr, uid,linea, ids, pay_amount, pay_account_id, period_id, pay_journal_id, writeoff_acc_id, writeoff_period_id, writeoff_journal_id, context=None, name=''):
        print "Metodo de conciliacion del asiento contable"
        line_obj = self.pool.get('account.retention.ica.line')      
        ret = self.browse(cr, uid, ids)[0] #account.retention.ica
        src_account_id=ret.account_id.id       
        if ret.type=="out_refund" or ret.type=="in_refund" :#para notas de credito
            types = {'out_invoice': -1, 'in_invoice': 1, 'out_refund': 1, 'in_refund': -1}
        else:
            types = {'out_invoice': 1, 'in_invoice': -1, 'out_refund': -1, 'in_refund': 1}
        direction = types[ret.type]
        #take the choosen date
        if 'date_p' in context and context['date_p']:
            date=context['date_p']
        else:
            date=time.strftime('%Y-%m-%d')
        l2 = {
            'debit': direction * pay_amount<0 and - direction * pay_amount     ,
            'credit': direction * pay_amount>0 and direction * pay_amount      ,
            'account_id': pay_account_id                                       , #cuenta de la cabecera
            'partner_id': ret.partner_id.id                                    ,
            'ref':ret.code                                                   ,
            'date': date                                                       ,
        }
        if not name:
            if ret.type in ['in_invoice','in_refund']:
                name = 'COMP. RET. ICA ' + ret.code + ' Doc. '+ (ret.invoice_id.reference or '')
            else:
                name = 'COMP. RET. ICA ' + ret.code + ' Doc. '+ (ret.invoice_id.number or '')
        l2['name'] = name
        lines = (0, 0, l2)        
        linea.append(lines) #se une la cabecera con las lineas del modelo para luego generar el asiento
        move = {'ref': ret.invoice_id.number, 'line_id': linea, 'journal_id': pay_journal_id, 'period_id': period_id, 'date': date}
        move_id = self.pool.get('account.move').create(cr, uid, move, context=context)
        self.pool.get('account.move').post(cr, uid, [move_id])
        line_ids = []
        total = 0.0
        line = self.pool.get('account.move.line')
        cr.execute('select id from account_move_line where move_id in ('+str(move_id)+','+str(ret.invoice_id.move_id.id)+')')
        line2 = line.browse(cr, uid, map(lambda x: x[0], cr.fetchall()) )
        for l in line2+ret.invoice_id.payment_ids:
            if l.account_id.id==src_account_id:
                line_ids.append(l.id)
                total += (l.debit or 0.0) - (l.credit or 0.0)
        if (not round(total,int(config['price_accuracy']))) or writeoff_acc_id:
            self.pool.get('account.move.line').reconcile(cr, uid, line_ids, 'manual', writeoff_acc_id, writeoff_period_id, writeoff_journal_id, context)
        else:
            self.pool.get('account.move.line').reconcile_partial(cr, uid, line_ids, 'manual', context)
        return {'move_id': move_id}



    def write(self, cr, uid, ids, vals, context=None, check=True, update_check=True):
        if not context:
            context={}
        return super(account_retention_ica, self).write(cr, uid, ids, vals, context=context)


    def unlink(self, cr, uid, ids, context=None):
        account_retention_ica = self.read(cr, uid, ids, ['state', 'invoice_id'])
        unlink_ids = []
        for t in account_retention_ica:
            if t['state'] in ('draft'):
                obj = self.pool.get('account.invoice') 
                invoice_brw = obj.browse(cr, uid, t['invoice_id'][0], context=None) 
                invoice_brw.write({'status_ica':"no_cal",'procesado_ica':"no_pro" }) 
                unlink_ids.append(t['id'])
            else:
                raise osv.except_osv(_('Invalida Accion!'), _('No se puede borrar este registro, esta en estado Realizado!'))
        osv.osv.unlink(self, cr, uid, unlink_ids, context=context)
        return True


account_retention_ica()

########################################################## LINE#######################################################
class account_retention_ica_line(osv.osv):
    
    
    _name = "account.retention.ica.line"
    _description = "Linea de Retencion"
    _columns = {
        'name': fields.char('Descripcion', size=64, required=False)                                                             ,
        'retention_id': fields.many2one('account.retention.ica', 'Comprobante', ondelete='cascade', required=True, readonly=False),
        'move_line_id': fields.many2one('account.move.line', 'Movimiento Contable', readonly=False, help="Asiento Contable")    , 
        'tarifa': fields.float('Tarifa %o.' ,digits=(16,2) , required=True, readonly=False)                                     , 
        'ciudad': fields.char('Ciudad', size=64, required=True)                                                                 ,
        'suma_total': fields.float('Suma' ,digits=(16,2) , required=True, readonly=False)                                       ,
        'account_id':fields.many2one('account.account','Cuenta Contable', required=True, domain="[('type', '<>', 'view')]")     ,
                                                                         
               } 
account_retention_ica_line()




